local ADDON_NAME, BigBrother = ...

local module = BigBrother:NewModule("Config")
local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")
local BuffCheck = BigBrother:GetModule("BuffCheck")

local GetChatChannels
do
	local channels = {}
	local function updateChannels()
		wipe(channels)
		for i = 1, GetNumDisplayChannels() do
			local name, _, _, _, _, _, category = GetChannelDisplayInfo(i)
			if category == "CHANNEL_CATEGORY_CUSTOM" then
				channels[name] = name
			end
		end
	end

	function GetChatChannels()
		return channels
	end
	BigBrother.GetChatChannels = GetChatChannels

	function module:CHANNEL_UI_UPDATE()
		updateChannels()
		LibStub("AceConfigRegistry-3.0"):NotifyChange("BigBrother")
	end
end

local function GetOptions()
	module:CHANNEL_UI_UPDATE()

	local db = BigBrother.db.profile
	local options = {
		name = ADDON_NAME,
		type = "group",
		childGroups = "tab",
		args = {
			confdesc = {
				name = GetAddOnMetadata("BigBrother", "Notes").."\n",
				type = "description",
				order = 1,
				width = "full",
			},
			quickcheck = {
				name = L["Quick Check"],
				desc = L["A quick report that shows who is missing consumables and/or buffs."],
				type = "execute",
				func = function() BigBrother:ConsumableCheck() end,
				order = 5,
				width = "full",
			},
			checks = {
				type = "group",
				name = L["Checks"],
				get = function(info) return db[info[#info]] end,
				set = function(info, value) db[info[#info]] = value end,
				order = 1,
				args = {
					checks = {
						name = L["Checks"],
						desc = L["Set what is included in a Quick Check."],
						type = "group",
						inline = true,
						order = 10,
						args = {
							checkFlasks = {
								name = L["Flasks/Elixirs"],
								desc = L["Include flasks and elixirs in checks."],
								type = "toggle",
								order = 5,
							},
							checkFood = {
								name = L["Food Buffs"],
								desc = L["Include food buffs in checks."],
								type = "toggle",
								order = 15,
							},
							checkBuffs = {
								name = L["Raid Buffs"],
								desc = L["Include raid buffs in checks."],
								type = "toggle",
								order = 20,
							},
						},
					}, --checks
					groups = {
						name = L["Checked Raid Groups"],
						desc = L["Set which raid groups are checked for buffs."],
						type = "group",
						inline = true,
						order = 20,
						disabled = function() return db.autogroups end,
						args = {
							autogroups = {
								name = L["Use Instance Difficulty"],
								desc = L["Use the currently selected instance difficulty to determine which groups to check."],
								type = "toggle",
								get = function() return db.autogroups end,
								set = function(info, value) db.autogroups = value end,
								width = "full",
								order = 1,
								disabled = false,
							},
							group1 = {
								name = L["Group 1"],
								type = "toggle",
								get = function() return db.groups[1] end,
								set = function(info, value) db.groups[1] = value end,
								order = 11,
							},
							group2 = {
								name = L["Group 2"],
								type = "toggle",
								get = function() return db.groups[2] end,
								set = function(info, value) db.groups[2] = value end,
								order = 12,
							},
							group3 = {
								name = L["Group 3"],
								type = "toggle",
								get = function() return db.groups[3] end,
								set = function(info, value) db.groups[3] = value end,
								order = 13,
							},
							group4 = {
								name = L["Group 4"],
								type = "toggle",
								get = function() return db.groups[4] end,
								set = function(info, value) db.groups[4] = value end,
								order = 14,
							},
							group5 = {
								name = L["Group 5"],
								type = "toggle",
								get = function() return db.groups[5] end,
								set = function(info, value) db.groups[5] = value end,
								order = 15,
							},
							group6 = {
								name = L["Group 6"],
								type = "toggle",
								get = function() return db.groups[6] end,
								set = function(info, value) db.groups[6] = value end,
								order = 16,
							},
							group7 = {
								name = L["Group 7"],
								type = "toggle",
								get = function() return db.groups[7] end,
								set = function(info, value) db.groups[7] = value end,
								order = 17,
							},
							group8 = {
								name = L["Group 8"],
								type = "toggle",
								get = function() return db.groups[8] end,
								set = function(info, value) db.groups[8] = value end,
								order = 18,
							},
						},
					}, --groups

					output = {
						name = L["Default Output"],
						desc = L["Set the default quick check output."],
						type = "group",
						inline = true,
						get = function(info)
							local key = info[#info]
							return string.gsub(db.outputDefault, "(.-):(.*)", "%1") == key
						end,
						set = function(info, value)
							if value then
								local key = info[#info]
								db.outputDefault = key
							end
						end,
						order = 20,
						args = {
							self = {
								name = L["Self"],
								desc = L["Print to your chat window."],
								type = "toggle",
								order = 5,
							},
							say = {
								name = L["Say"],
								desc = L["Report to say."],
								type = "toggle",
								order = 8,
							},
							group = {
								name = L["Group"],
								desc = L["Report to party or raid chat."],
								type = "toggle",
								order = 10,
							},
							party = {
								name = L["Party"],
								desc = L["Report to party chat."],
								type = "toggle",
								order = 11,
							},
							raid = {
								name = L["Raid"],
								desc = L["Report to raid chat."],
								type = "toggle",
								order = 12,
							},
							raid_warning = {
								name = L["Raid Warning"],
								desc = L["Report as a raid warning."],
								type = "toggle",
								order = 13,
							},
							guild = {
								name = L["Guild"],
								desc = L["Report to guild chat."],
								type = "toggle",
								order = 15,
							},
							officer = {
								name = L["Officer"],
								desc = L["Report to officer chat."],
								type = "toggle",
								order = 20,
							},
							blame = {
								name = L["Blame"],
								desc = L["Whisper players that are missing consumables."],
								type = "toggle",
								order = 23,
							},
							channel = {
								name = L["Channel"],
								desc = L["Report to a custom channel."],
								type = "toggle",
								order = 25,
							},
							extra = { --pretty silly implementation, but oh well
								name = L["Channel"],
								type = "select",
								values = GetChatChannels,
								get = function()
									local channel = string.gsub(db.outputDefault, "(.-):(.*)", "%2")
									return GetChannelName(channel) > 0 and channel or ""
								end,
								set = function(info, value)
									db.outputDefault = "channel:"..value
								end,
								hidden = function() return string.gsub(db.outputDefault, "(.-):(.*)", "%1") ~= "channel" end,
								order = 101,
							},
						},
					},

					checkReadyCheck = {
						name = L["Check on Ready Check"],
						desc = L["Quick Check on a ready check when in a non-LFG instance group.\n\n|cffffff33Checked|r: Report if started by you.\n|cffccccccGray|r: Report always."],
						type = "toggle",
						tristate = true,
						-- unchecked (false), checked (true), greyed (nil)
						-- unchecked (0), checked (1), greyed (2)
						get = function()
							local value = db.checkReadyCheck
							if value == 0 then
								return false
							elseif value == 1 then
								return true
							elseif value == 2 then
								return nil
							end
						end,
						set = function(info, value)
							if value == false then
								value = 0
							elseif value == true then
								value = 1
							elseif value == nil then
								value = 2
							end
							db.checkReadyCheck = value
						end,
						order = 30,
					},
					maxOnly = {
						name = L["Max Quality Only"],
						desc = L["Only count consumables of the highest available stat value in checks."],
						type = "toggle",
						order = 35,
					},
				},
			}, --checks

		},
	}

	for _, module in BigBrother:IterateModules() do
		if module.AddOptions then
			module:AddOptions(options)
		end
	end

	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(BigBrother.db)
	options.args.profile.desc = nil
	options.args.profile.order = 100

	return options
end

function module:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BigBrother", GetOptions)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BigBrother")

	--LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("BigBrother Profile", LibStub("AceDBOptions-3.0"):GetOptionsTable(BigBrother.db))
	--LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BigBrother Profile", "Profiles", "BigBrother")
end

function module:OnEnable()
	self:RegisterEvent("CHANNEL_UI_UPDATE")
end

function module:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

local function ChatCommand(input)
	local command, rest = string.match(input, "^(%S*)%s*(.-)$")
	if command == "quickcheck" or command == "qc" then
		BigBrother:ConsumableCheck(rest ~= "" and rest)
	elseif command == "buffs" then
		BuffCheck:ToggleWindow()
	elseif command == "config" then
		module:OpenOptions()
	else
		BigBrother:Print("|cffffff7f/bb quickcheck [self||group||party||raid||guild||officer||blame]|r: " .. L["A quick report that shows who is missing consumables and/or buffs."])
		BigBrother:Print("|cffffff7f/bb buffs|r: " .. L["Opens the buff window (drag the bottom to resize)."])
		BigBrother:Print("|cffffff7f/bb config|r: " .. L["Open settings."])
	end
end

SLASH_BIGBROTHER1 = "/bigbrother"
SLASH_BIGBROTHER2 = "/bb"
SlashCmdList["BIGBROTHER"] = ChatCommand
